/*******************************************************************************
* File Name: WrNFC_UartINT.c
* Version 2.50
*
* Description:
*  This file provides all Interrupt Service functionality of the UART component
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "WrNFC_Uart.h"
#include "cyapicallbacks.h"


/***************************************
* Custom Declarations
***************************************/
/* `#START CUSTOM_DECLARATIONS` Place your declaration here */

/* `#END` */

#if (WrNFC_Uart_RX_INTERRUPT_ENABLED && (WrNFC_Uart_RX_ENABLED || WrNFC_Uart_HD_ENABLED))
    /*******************************************************************************
    * Function Name: WrNFC_Uart_RXISR
    ********************************************************************************
    *
    * Summary:
    *  Interrupt Service Routine for RX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  WrNFC_Uart_rxBuffer - RAM buffer pointer for save received data.
    *  WrNFC_Uart_rxBufferWrite - cyclic index for write to rxBuffer,
    *     increments after each byte saved to buffer.
    *  WrNFC_Uart_rxBufferRead - cyclic index for read from rxBuffer,
    *     checked to detect overflow condition.
    *  WrNFC_Uart_rxBufferOverflow - software overflow flag. Set to one
    *     when WrNFC_Uart_rxBufferWrite index overtakes
    *     WrNFC_Uart_rxBufferRead index.
    *  WrNFC_Uart_rxBufferLoopDetect - additional variable to detect overflow.
    *     Set to one when WrNFC_Uart_rxBufferWrite is equal to
    *    WrNFC_Uart_rxBufferRead
    *  WrNFC_Uart_rxAddressMode - this variable contains the Address mode,
    *     selected in customizer or set by UART_SetRxAddressMode() API.
    *  WrNFC_Uart_rxAddressDetected - set to 1 when correct address received,
    *     and analysed to store following addressed data bytes to the buffer.
    *     When not correct address received, set to 0 to skip following data bytes.
    *
    *******************************************************************************/
    CY_ISR(WrNFC_Uart_RXISR)
    {
        uint8 readData;
        uint8 readStatus;
        uint8 increment_pointer = 0u;

    #if(CY_PSOC3)
        uint8 int_en;
    #endif /* (CY_PSOC3) */

    #ifdef WrNFC_Uart_RXISR_ENTRY_CALLBACK
        WrNFC_Uart_RXISR_EntryCallback();
    #endif /* WrNFC_Uart_RXISR_ENTRY_CALLBACK */

        /* User code required at start of ISR */
        /* `#START WrNFC_Uart_RXISR_START` */

        /* `#END` */

    #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
        int_en = EA;
        CyGlobalIntEnable;
    #endif /* (CY_PSOC3) */

        do
        {
            /* Read receiver status register */
            readStatus = WrNFC_Uart_RXSTATUS_REG;
            /* Copy the same status to readData variable for backward compatibility support 
            *  of the user code in WrNFC_Uart_RXISR_ERROR` section. 
            */
            readData = readStatus;

            if((readStatus & (WrNFC_Uart_RX_STS_BREAK | 
                            WrNFC_Uart_RX_STS_PAR_ERROR |
                            WrNFC_Uart_RX_STS_STOP_ERROR | 
                            WrNFC_Uart_RX_STS_OVERRUN)) != 0u)
            {
                /* ERROR handling. */
                WrNFC_Uart_errorStatus |= readStatus & ( WrNFC_Uart_RX_STS_BREAK | 
                                                            WrNFC_Uart_RX_STS_PAR_ERROR | 
                                                            WrNFC_Uart_RX_STS_STOP_ERROR | 
                                                            WrNFC_Uart_RX_STS_OVERRUN);
                /* `#START WrNFC_Uart_RXISR_ERROR` */

                /* `#END` */
                
            #ifdef WrNFC_Uart_RXISR_ERROR_CALLBACK
                WrNFC_Uart_RXISR_ERROR_Callback();
            #endif /* WrNFC_Uart_RXISR_ERROR_CALLBACK */
            }
            
            if((readStatus & WrNFC_Uart_RX_STS_FIFO_NOTEMPTY) != 0u)
            {
                /* Read data from the RX data register */
                readData = WrNFC_Uart_RXDATA_REG;
            #if (WrNFC_Uart_RXHW_ADDRESS_ENABLED)
                if(WrNFC_Uart_rxAddressMode == (uint8)WrNFC_Uart__B_UART__AM_SW_DETECT_TO_BUFFER)
                {
                    if((readStatus & WrNFC_Uart_RX_STS_MRKSPC) != 0u)
                    {
                        if ((readStatus & WrNFC_Uart_RX_STS_ADDR_MATCH) != 0u)
                        {
                            WrNFC_Uart_rxAddressDetected = 1u;
                        }
                        else
                        {
                            WrNFC_Uart_rxAddressDetected = 0u;
                        }
                    }
                    if(WrNFC_Uart_rxAddressDetected != 0u)
                    {   /* Store only addressed data */
                        WrNFC_Uart_rxBuffer[WrNFC_Uart_rxBufferWrite] = readData;
                        increment_pointer = 1u;
                    }
                }
                else /* Without software addressing */
                {
                    WrNFC_Uart_rxBuffer[WrNFC_Uart_rxBufferWrite] = readData;
                    increment_pointer = 1u;
                }
            #else  /* Without addressing */
                WrNFC_Uart_rxBuffer[WrNFC_Uart_rxBufferWrite] = readData;
                increment_pointer = 1u;
            #endif /* (WrNFC_Uart_RXHW_ADDRESS_ENABLED) */

                /* Do not increment buffer pointer when skip not addressed data */
                if(increment_pointer != 0u)
                {
                    if(WrNFC_Uart_rxBufferLoopDetect != 0u)
                    {   /* Set Software Buffer status Overflow */
                        WrNFC_Uart_rxBufferOverflow = 1u;
                    }
                    /* Set next pointer. */
                    WrNFC_Uart_rxBufferWrite++;

                    /* Check pointer for a loop condition */
                    if(WrNFC_Uart_rxBufferWrite >= WrNFC_Uart_RX_BUFFER_SIZE)
                    {
                        WrNFC_Uart_rxBufferWrite = 0u;
                    }

                    /* Detect pre-overload condition and set flag */
                    if(WrNFC_Uart_rxBufferWrite == WrNFC_Uart_rxBufferRead)
                    {
                        WrNFC_Uart_rxBufferLoopDetect = 1u;
                        /* When Hardware Flow Control selected */
                        #if (WrNFC_Uart_FLOW_CONTROL != 0u)
                            /* Disable RX interrupt mask, it is enabled when user read data from the buffer using APIs */
                            WrNFC_Uart_RXSTATUS_MASK_REG  &= (uint8)~WrNFC_Uart_RX_STS_FIFO_NOTEMPTY;
                            CyIntClearPending(WrNFC_Uart_RX_VECT_NUM);
                            break; /* Break the reading of the FIFO loop, leave the data there for generating RTS signal */
                        #endif /* (WrNFC_Uart_FLOW_CONTROL != 0u) */
                    }
                }
            }
        }while((readStatus & WrNFC_Uart_RX_STS_FIFO_NOTEMPTY) != 0u);

        /* User code required at end of ISR (Optional) */
        /* `#START WrNFC_Uart_RXISR_END` */

        /* `#END` */

    #ifdef WrNFC_Uart_RXISR_EXIT_CALLBACK
        WrNFC_Uart_RXISR_ExitCallback();
    #endif /* WrNFC_Uart_RXISR_EXIT_CALLBACK */

    #if(CY_PSOC3)
        EA = int_en;
    #endif /* (CY_PSOC3) */
    }
    
#endif /* (WrNFC_Uart_RX_INTERRUPT_ENABLED && (WrNFC_Uart_RX_ENABLED || WrNFC_Uart_HD_ENABLED)) */


#if (WrNFC_Uart_TX_INTERRUPT_ENABLED && WrNFC_Uart_TX_ENABLED)
    /*******************************************************************************
    * Function Name: WrNFC_Uart_TXISR
    ********************************************************************************
    *
    * Summary:
    * Interrupt Service Routine for the TX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  WrNFC_Uart_txBuffer - RAM buffer pointer for transmit data from.
    *  WrNFC_Uart_txBufferRead - cyclic index for read and transmit data
    *     from txBuffer, increments after each transmitted byte.
    *  WrNFC_Uart_rxBufferWrite - cyclic index for write to txBuffer,
    *     checked to detect available for transmission bytes.
    *
    *******************************************************************************/
    CY_ISR(WrNFC_Uart_TXISR)
    {
    #if(CY_PSOC3)
        uint8 int_en;
    #endif /* (CY_PSOC3) */

    #ifdef WrNFC_Uart_TXISR_ENTRY_CALLBACK
        WrNFC_Uart_TXISR_EntryCallback();
    #endif /* WrNFC_Uart_TXISR_ENTRY_CALLBACK */

        /* User code required at start of ISR */
        /* `#START WrNFC_Uart_TXISR_START` */

        /* `#END` */

    #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
        int_en = EA;
        CyGlobalIntEnable;
    #endif /* (CY_PSOC3) */

        while((WrNFC_Uart_txBufferRead != WrNFC_Uart_txBufferWrite) &&
             ((WrNFC_Uart_TXSTATUS_REG & WrNFC_Uart_TX_STS_FIFO_FULL) == 0u))
        {
            /* Check pointer wrap around */
            if(WrNFC_Uart_txBufferRead >= WrNFC_Uart_TX_BUFFER_SIZE)
            {
                WrNFC_Uart_txBufferRead = 0u;
            }

            WrNFC_Uart_TXDATA_REG = WrNFC_Uart_txBuffer[WrNFC_Uart_txBufferRead];

            /* Set next pointer */
            WrNFC_Uart_txBufferRead++;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START WrNFC_Uart_TXISR_END` */

        /* `#END` */

    #ifdef WrNFC_Uart_TXISR_EXIT_CALLBACK
        WrNFC_Uart_TXISR_ExitCallback();
    #endif /* WrNFC_Uart_TXISR_EXIT_CALLBACK */

    #if(CY_PSOC3)
        EA = int_en;
    #endif /* (CY_PSOC3) */
   }
#endif /* (WrNFC_Uart_TX_INTERRUPT_ENABLED && WrNFC_Uart_TX_ENABLED) */


/* [] END OF FILE */
