/*******************************************************************************
* File Name: WrNFC_Uart_PM.c
* Version 2.50
*
* Description:
*  This file provides Sleep/WakeUp APIs functionality.
*
* Note:
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "WrNFC_Uart.h"


/***************************************
* Local data allocation
***************************************/

static WrNFC_Uart_BACKUP_STRUCT  WrNFC_Uart_backup =
{
    /* enableState - disabled */
    0u,
};



/*******************************************************************************
* Function Name: WrNFC_Uart_SaveConfig
********************************************************************************
*
* Summary:
*  This function saves the component nonretention control register.
*  Does not save the FIFO which is a set of nonretention registers.
*  This function is called by the WrNFC_Uart_Sleep() function.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  WrNFC_Uart_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void WrNFC_Uart_SaveConfig(void)
{
    #if(WrNFC_Uart_CONTROL_REG_REMOVED == 0u)
        WrNFC_Uart_backup.cr = WrNFC_Uart_CONTROL_REG;
    #endif /* End WrNFC_Uart_CONTROL_REG_REMOVED */
}


/*******************************************************************************
* Function Name: WrNFC_Uart_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the nonretention control register except FIFO.
*  Does not restore the FIFO which is a set of nonretention registers.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  WrNFC_Uart_backup - used when non-retention registers are restored.
*
* Reentrant:
*  No.
*
* Notes:
*  If this function is called without calling WrNFC_Uart_SaveConfig() 
*  first, the data loaded may be incorrect.
*
*******************************************************************************/
void WrNFC_Uart_RestoreConfig(void)
{
    #if(WrNFC_Uart_CONTROL_REG_REMOVED == 0u)
        WrNFC_Uart_CONTROL_REG = WrNFC_Uart_backup.cr;
    #endif /* End WrNFC_Uart_CONTROL_REG_REMOVED */
}


/*******************************************************************************
* Function Name: WrNFC_Uart_Sleep
********************************************************************************
*
* Summary:
*  This is the preferred API to prepare the component for sleep. 
*  The WrNFC_Uart_Sleep() API saves the current component state. Then it
*  calls the WrNFC_Uart_Stop() function and calls 
*  WrNFC_Uart_SaveConfig() to save the hardware configuration.
*  Call the WrNFC_Uart_Sleep() function before calling the CyPmSleep() 
*  or the CyPmHibernate() function. 
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  WrNFC_Uart_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void WrNFC_Uart_Sleep(void)
{
    #if(WrNFC_Uart_RX_ENABLED || WrNFC_Uart_HD_ENABLED)
        if((WrNFC_Uart_RXSTATUS_ACTL_REG  & WrNFC_Uart_INT_ENABLE) != 0u)
        {
            WrNFC_Uart_backup.enableState = 1u;
        }
        else
        {
            WrNFC_Uart_backup.enableState = 0u;
        }
    #else
        if((WrNFC_Uart_TXSTATUS_ACTL_REG  & WrNFC_Uart_INT_ENABLE) !=0u)
        {
            WrNFC_Uart_backup.enableState = 1u;
        }
        else
        {
            WrNFC_Uart_backup.enableState = 0u;
        }
    #endif /* End WrNFC_Uart_RX_ENABLED || WrNFC_Uart_HD_ENABLED*/

    WrNFC_Uart_Stop();
    WrNFC_Uart_SaveConfig();
}


/*******************************************************************************
* Function Name: WrNFC_Uart_Wakeup
********************************************************************************
*
* Summary:
*  This is the preferred API to restore the component to the state when 
*  WrNFC_Uart_Sleep() was called. The WrNFC_Uart_Wakeup() function
*  calls the WrNFC_Uart_RestoreConfig() function to restore the 
*  configuration. If the component was enabled before the 
*  WrNFC_Uart_Sleep() function was called, the WrNFC_Uart_Wakeup()
*  function will also re-enable the component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  WrNFC_Uart_backup - used when non-retention registers are restored.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void WrNFC_Uart_Wakeup(void)
{
    WrNFC_Uart_RestoreConfig();
    #if( (WrNFC_Uart_RX_ENABLED) || (WrNFC_Uart_HD_ENABLED) )
        WrNFC_Uart_ClearRxBuffer();
    #endif /* End (WrNFC_Uart_RX_ENABLED) || (WrNFC_Uart_HD_ENABLED) */
    #if(WrNFC_Uart_TX_ENABLED || WrNFC_Uart_HD_ENABLED)
        WrNFC_Uart_ClearTxBuffer();
    #endif /* End WrNFC_Uart_TX_ENABLED || WrNFC_Uart_HD_ENABLED */

    if(WrNFC_Uart_backup.enableState != 0u)
    {
        WrNFC_Uart_Enable();
    }
}


/* [] END OF FILE */
